@@ -7,8 +7,12 @@ use XML::DOM;
 #			Interface Definition Language (OMG IDL CORBA v3.0)
 #
 
+use CORBA::XMLSchemas::xsd;
+
 package CORBA::XMLSchemas::relaxngVisitor;
 
+use base qw(CORBA::XMLSchemas::baseVisitor);
+
 # needs $node->{xsd_name} (XsdNameVisitor)
 
 use File::Basename;
@@ -20,14 +24,19 @@ sub new {
 	bless($self, $class);
 	my ($parser, $standalone, $tag_root) = @_;
 	$self->{standalone} = $standalone;
+	$self->{beautify} = $parser->YYData->{opt_t}; 
 	$self->{tag_root} = $tag_root || "";
-	$self->{rng} = 'rng';
-	$self->{xsd} = 'xs';
-#	$self->{corba} = 'corba';
+	$self->{_rng} = 'rng';
+	$self->{rng} = $parser->YYData->{opt_q} ? 'rng:' : '';
+	$self->{_xsd} = 'xs';
+	$self->{xsd} = 'xs:';
+	$self->{_corba} = 'corba';
+	$self->{corba} = 'corba:';
 	$self->{srcname} = $parser->YYData->{srcname};
 	$self->{srcname_size} = $parser->YYData->{srcname_size};
 	$self->{srcname_mtime} = $parser->YYData->{srcname_mtime};
 	$self->{symbtab} = $parser->YYData->{symbtab};
+	$self->{base} = $parser->YYData->{opt_b} || "";
 	$self->{root} = $parser->YYData->{root};
 	my $filename = basename($self->{srcname}, ".idl") . ".rng";
 	$self->open_stream($filename);
@@ -37,70 +46,6 @@ sub new {
 	return $self;
 }
 
-sub open_stream {
-	my $self = shift;
-	my ($filename) = @_;
-	open(OUT, "> $filename")
-			or die "can't open $filename ($!).\n";
-	$self->{out} = \*OUT;
-	$self->{filename} = $filename;
-}
-
-sub _value {
-	my $self = shift;
-	my ($node) = @_;
-
-	my $value = $node->{value};
-	if (ref $value and $value->isa('Enum')) {
-		return $value->{xsd_name};
-	} else {
-		my $str = $value;
-		$str =~ s/^\+//;
-		return $str;
-	}
-}
-
-sub _beautify {
-	my $self = shift;
-	my ($in) = @_;
-	my $out = '';
-	my @tab;
-	foreach (split /(<[^>']*(?:'[^']*'[^>']*)*>)/, $in) {
-		next unless ($_);
-		pop @tab if (/^<\//);
-		if (/^<\// and $out !~ /\s$/) {
-			$out .= "$_\n";
-		} elsif (/^</) {
-			$out .= join('', @tab) . "$_\n";
-		} else {
-			$out =~ s/\s+$//;
-			$out .= $_;
-		}
-		push @tab, "  " if (/^<[^\/?!]/ and /[^\/]>$/);
-	}
-	$out =~ s/\s+$//;
-	return $out;
-}
-
-sub _no_mapping {
-	my $self = shift;
-	my ($node, $dom_parent) = @_;
-
-	my $str = " no mapping for " . $node->{full} . " (" . ref $node . ") ";
-	my $comment = $self->{dom_doc}->createComment($str);
-	$dom_parent->appendChild($comment);
-}
-
-sub _get_defn {
-	my $self = shift;
-	my ($defn) = @_;
-	if (ref $defn) {
-		return $defn;
-	} else {
-		return $self->{symbtab}->Lookup($defn);
-	}
-}
-
 sub _ref_type {
 	my $self = shift;
 	my ($type, $dom_parent) = @_;
@@ -110,11 +55,11 @@ sub _ref_type {
 			or $type->isa('UnionType')
 			or $type->isa('EnumType')
 			or $type->isa('BaseInterface') ) {
-		my $ref = $self->{dom_doc}->createElement($self->{rng} . ":ref");
+		my $ref = $self->{dom_doc}->createElement($self->{rng} . "ref");
 		$ref->setAttribute("name", $type->{xsd_name});
 		$dom_parent->appendChild($ref);
 	} else {
-		my $data = $self->{dom_doc}->createElement($self->{rng} . ":data");
+		my $data = $self->{dom_doc}->createElement($self->{rng} . "data");
 		$data->setAttribute("type", $type->{xsd_name});
 		$dom_parent->appendChild($data);
 
@@ -126,29 +71,34 @@ sub _standalone {
 	my $self = shift;
 	my ($node, $dom_parent) = @_;
 
-	if ($self->{tag_root} eq $node->{xsd_name}) {
-		$self->{dom_start} = $self->{dom_doc}->createElement($self->{rng} . ":start");
+	if (       $self->{tag_root} eq $node->{xsd_name}
+			or $node->hasProperty("start") ) {
+		$self->{dom_start} = $self->{dom_doc}->createElement($self->{rng} . "start");
 
-		my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
-		$element->setAttribute("name", $node->{xsd_name});
+		my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
+		if ($node->hasProperty("start")) {
+			$element->setAttribute("name", $node->getProperty("start"));
+		} else {
+			$element->setAttribute("name", $node->{xsd_name});
+		}
 		$self->{dom_start}->appendChild($element);
 
-		my $ref = $self->{dom_doc}->createElement($self->{rng} . ":ref");
+		my $ref = $self->{dom_doc}->createElement($self->{rng} . "ref");
 		$ref->setAttribute("name", $node->{xsd_name});
 		$element->appendChild($ref);
 	} elsif ($self->{standalone}) {
-		my $div = $self->{dom_doc}->createElement($self->{rng} . ":div");
+		my $div = $self->{dom_doc}->createElement($self->{rng} . "div");
 		$dom_parent->appendChild($div);
 
-		my $start = $self->{dom_doc}->createElement($self->{rng} . ":start");
+		my $start = $self->{dom_doc}->createElement($self->{rng} . "start");
 		$start->setAttribute("combine", "choice");
 		$div->appendChild($start);
 
-		my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+		my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 		$element->setAttribute("name", $node->{xsd_name});
 		$start->appendChild($element);
 
-		my $ref = $self->{dom_doc}->createElement($self->{rng} . ":ref");
+		my $ref = $self->{dom_doc}->createElement($self->{rng} . "ref");
 		$ref->setAttribute("name", $node->{xsd_name});
 		$element->appendChild($ref);
 	}
@@ -166,20 +116,24 @@ sub visitSpecification {
 	$self->{dom_doc} = new XML::DOM::Document();
 	$self->{dom_parent} = $self->{dom_doc};
 
-	my $grammar = $self->{dom_doc}->createElement($self->{rng} . ":grammar");
+	my $grammar = $self->{dom_doc}->createElement($self->{rng} . "grammar");
 	$grammar->setAttribute("ns", "http://www.omg.org/IDL-Mapped/");
 	$grammar->setAttribute("datatypeLibrary", "http://www.w3.org/2001/XMLSchema-datatypes");
-	$grammar->setAttribute("xmlns:" . $self->{rng}, "http://relaxng.org/ns/structure/1.0");
+	$grammar->setAttribute("xmlns:" . $self->{_xsd}, "http://www.w3.org/2001/XMLSchema");
+	if ($self->{rng}) {
+		$grammar->setAttribute("xmlns:" . $self->{_rng}, "http://relaxng.org/ns/structure/1.0");
+	} else {
+		$grammar->setAttribute("xmlns", "http://relaxng.org/ns/structure/1.0");
+	}
+	$grammar->setAttribute("xmlns:" . $self->{_corba}, "http://www.omg.org/IDL-WSDL/1.0/");
 	$self->{dom_parent}->appendChild($grammar);
 
-	if ($self->{root}->{need_corba}) {
-		my $include = $self->{dom_doc}->createElement($self->{xsd} . ":include");
-		$include->setAttribute("href", "http://www.omg.org/IDL-WSDL/1.0/");
+	if ($self->{root}->{need_corba} or $self->{root}->{need_any}) {
+		my $include = $self->{dom_doc}->createElement($self->{xsd} . "include");
+		$include->setAttribute("href", $self->{base} . "corba.rng");
 		$grammar->appendChild($include);
 	}
 
-	$self->_any($grammar);
-
 	if (exists $node->{list_import}) {
 		foreach (@{$node->{list_import}}) {
 			$_->visit($self, $grammar);
@@ -191,120 +145,21 @@ sub visitSpecification {
 
 	$grammar->appendChild($self->{dom_start}) if (exists $self->{dom_start});
 
-	print $FH "<!-- This file was generated (by ",$0,"). DO NOT modify it -->\n";
-	print $FH "<!-- From file : ",$self->{srcname},", ",$self->{srcname_size}," octets, ",POSIX::ctime($self->{srcname_mtime});
-	print $FH "-->\n";
-	print $FH "\n";
-	print $FH $self->_beautify($self->{dom_doc}->toString());
-	print $FH "\n\n";
-	print $FH "<!-- end of file : ",$self->{filename}," -->\n";
+	if ($self->{beautify}) {
+		print $FH "<!-- This file was generated (by ",$0,"). DO NOT modify it -->\n";
+		print $FH "<!-- From file : ",$self->{srcname},", ",$self->{srcname_size}," octets, ",POSIX::ctime($self->{srcname_mtime});
+		print $FH "-->\n";
+		print $FH "\n";
+		print $FH $self->_beautify($self->{dom_doc}->toString());
+		print $FH "\n\n";
+		print $FH "<!-- end of file : ",$self->{filename}," -->\n";
+	} else {
+		print $FH $self->{dom_doc}->toString();
+	}
 	close $FH;
 	$self->{dom_doc}->dispose();
 }
 
-sub _any {
-	my $self = shift;
-	my ($dom_parent) = @_;
-
-	return unless ($self->{root}->{need_any});
-
-	#	See	1.2.7.7		TypeCode
-	my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
-	$define->setAttribute("name", "CORBA.TypeCode");
-	$dom_parent->appendChild($define);
-
-	my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
-	$define->appendChild($group);
-
-	my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
-	$element->setAttribute("name", "definition");
-	$group->appendChild($element);
-
-	my $data = $self->{dom_doc}->createElement($self->{rng} . ":data");
-	$data->setAttribute("type", $self->{xsd} . ":url");
-	$element->appendChild($data);
-
-	$element = $self->{dom_doc}->createElement($self->{rng} . ":element");
-	$element->setAttribute("name", "typename");
-	$group->appendChild($element);
-
-	$data = $self->{dom_doc}->createElement($self->{rng} . ":data");
-	$data->setAttribute("type", $self->{xsd} . ":string");
-	$element->appendChild($data);
-
-	#	See	1.2.7.8		Any
-	$define = $self->{dom_doc}->createElement($self->{rng} . ":define");
-	$define->setAttribute("name", "CORBA.Any");
-	$dom_parent->appendChild($define);
-
-	$group = $self->{dom_doc}->createElement($self->{rng} . ":group");
-	$define->appendChild($group);
-
-	$element = $self->{dom_doc}->createElement($self->{rng} . ":element");
-	$element->setAttribute("name", "type");
-	$group->appendChild($element);
-
-	my $ref = $self->{dom_doc}->createElement($self->{rng} . ":ref");
-	$ref->setAttribute("name", "CORBA.TypeCode");
-	$element->appendChild($ref);
-
-	$element = $self->{dom_doc}->createElement($self->{rng} . ":element");
-	$element->setAttribute("name", "value");
-	$group->appendChild($element);
-
-	$data = $self->{dom_doc}->createElement($self->{rng} . ":data");
-	$data->setAttribute("type", $self->{xsd} . ":anyType");
-	$element->appendChild($data);
-
-	delete $self->{root}->{need_any};
-}
-
-#
-#	3.7		Module Declaration
-#
-
-sub visitModules {
-	my $self = shift;
-	my ($node, $dom_parent) = @_;
-	unless (exists $node->{$self->{num_key}}) {
-		$node->{$self->{num_key}} = 0;
-	}
-	my $module = ${$node->{list_decl}}[$node->{$self->{num_key}}];
-	$module->visit($self, $dom_parent);
-	$node->{$self->{num_key}} ++;
-}
-
-sub visitModule {
-	my $self = shift;
-	my ($node, $dom_parent) = @_;
-	foreach (@{$node->{list_decl}}) {
-		$self->_get_defn($_)->visit($self, $dom_parent);
-	}
-}
-
-#
-#	3.8		Interface Declaration
-#
-#	See 1.2.8		Interfaces
-#
-
-sub visitInterface {
-	my $self = shift;
-	my ($node, $dom_parent) = @_;
-
-	foreach (@{$node->{list_decl}}) {
-		$self->_get_defn($_)->visit($self, $dom_parent);
-	}
-}
-
-sub visitLocalInterface {
-	shift->_no_mapping(@_);
-}
-
-sub visitForwardBaseInterface {
-	# empty
-}
-
 #
 #	3.9		Value Declaration
 #
@@ -326,16 +181,25 @@ sub visitRegularValue {
 		$value_element->visit($self, $dom_parent);
 	}
 
-	my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+	my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 	$define->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($define);
 
-	my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
+	my $group = $self->{dom_doc}->createElement($self->{rng} . "group");
 	$define->appendChild($group);
 
-	foreach (values %{$node->{hash_attribute_operation}}) {
+	foreach ($node->getInheritance()) {
+		my $base = $self->_get_defn($_);
+		foreach (@{$base->{list_decl}}) {
+			my $defn = $self->_get_defn($_);
+			if ($defn->isa('StateMembers')) {
+				$defn->visit($self, $group);
+			}
+		}
+	}
+	foreach (@{$node->{list_decl}}) {
 		my $defn = $self->_get_defn($_);
-		if ($defn->isa('StateMember')) {
+		if ($defn->isa('StateMembers')) {
 			$defn->visit($self, $group);
 		}
 	}
@@ -349,14 +213,14 @@ sub _value_id {
 	my $self = shift;
 	my ($dom_parent) = @_;
 
-	my $optional = $self->{dom_doc}->createElement($self->{rng} . ":optional");
+	my $optional = $self->{dom_doc}->createElement($self->{rng} . "optional");
 	$dom_parent->appendChild($optional);
 
-	my $attribute = $self->{dom_doc}->createElement($self->{rng} . ":attribute");
+	my $attribute = $self->{dom_doc}->createElement($self->{rng} . "attribute");
 	$attribute->setAttribute("name", "id");
 	$optional->appendChild($attribute);
 
-	my $data = $self->{dom_doc}->createElement($self->{rng} . ":data");
+	my $data = $self->{dom_doc}->createElement($self->{rng} . "data");
 	$data->setAttribute("type", "ID");
 	$attribute->appendChild($data);
 }
@@ -375,17 +239,17 @@ sub visitStateMember {
 			$current = $self->_get_defn($current->{type});
 		}
 
-		my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+		my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 		$element->setAttribute("name", $node->{xsd_name});
 		$dom_parent->appendChild($element);
 
 		$current = $element;
 		my $first = 1;
 		foreach (reverse @{$node->{array_size}}) {
-			my $oneOrMore = $self->{dom_doc}->createElement($self->{rng} . ":oneOrMore");
+			my $oneOrMore = $self->{dom_doc}->createElement($self->{rng} . "oneOrMore");
 			$current->appendChild($oneOrMore);
 
-			my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+			my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 			my $item = ($idx != 0) ? "item" . $idx : "item";
 			$element->setAttribute("name", $item);
 			$oneOrMore->appendChild($element);
@@ -402,27 +266,27 @@ sub visitStateMember {
 		}
 	} else {
 		if ($type->isa('RegularValue')) {
-			my $choice = $self->{dom_doc}->createElement($self->{rng} . ":choice");
+			my $choice = $self->{dom_doc}->createElement($self->{rng} . "choice");
 			$dom_parent->appendChild($choice);
 
-			my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+			my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 			$element->setAttribute("name", $node->{xsd_name});
 			$choice->appendChild($element);
 
-			my $ref = $self->{dom_doc}->createElement($self->{rng} . ":ref");
+			my $ref = $self->{dom_doc}->createElement($self->{rng} . "ref");
 			$ref->setAttribute("name", $type->{xsd_name});
 			$element->appendChild($ref);
 
-			$element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+			$element = $self->{dom_doc}->createElement($self->{rng} . "element");
 			$element->setAttribute("name", "_REF_" . $node->{xsd_name});
 			$choice->appendChild($element);
 
-			$ref = $self->{dom_doc}->createElement($self->{rng} . ":ref");
+			$ref = $self->{dom_doc}->createElement($self->{rng} . "ref");
 			$ref->setAttribute("name", "_VALREF");
 			$element->appendChild($ref);
 		} else {
 			# like Single
-			my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+			my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 			$element->setAttribute("name", $node->{xsd_name});
 			$dom_parent->appendChild($element);
 
@@ -446,14 +310,14 @@ sub visitBoxedValue {
 		$type->visit($self, $dom_parent);
 	}
 
-	my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+	my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 	$define->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($define);
 
-	my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
+	my $group = $self->{dom_doc}->createElement($self->{rng} . "group");
 	$define->appendChild($group);
 
-	my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+	my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 	$element->setAttribute("name", "value");
 	$group->appendChild($element);
 
@@ -468,44 +332,12 @@ sub visitBoxedValue {
 	$self->_standalone($node, $dom_parent);
 }
 
-sub visitAbstractValue {
-	my $self = shift;
-	my ($node, $dom_parent) = @_;
-
-	foreach (@{$node->{list_decl}}) {
-		my $value_element = $self->_get_defn($_);
-		if (	   $value_element->isa('Operation')
-				or $value_element->isa('Attributes') ) {
-			next;
-		}
-		$value_element->visit($self, $dom_parent);
-	}
-}
-
-#
-#	3.10	Constant Declaration
-#
-#	See	1.2.6.1		Constants
-#
-
-sub visitConstant {
-	# empty
-}
-
 #
 #	3.11	Type Declaration
 #
 #	See	1.2.7.3		Typedefs
 #
 
-sub visitTypeDeclarators {
-	my $self = shift;
-	my ($node, $dom_parent) = @_;
-	foreach (@{$node->{list_decl}}) {
-		$self->_get_defn($_)->visit($self, $dom_parent);
-	}
-}
-
 sub visitTypeDeclarator {
 	my $self = shift;
 	my ($node, $dom_parent) = @_;
@@ -517,7 +349,7 @@ sub visitTypeDeclarator {
 		$type->visit($self, $dom_parent);
 	}
 
-	my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+	my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 	$define->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($define);
 
@@ -538,10 +370,10 @@ sub visitTypeDeclarator {
 		$current = $define;
 		my $first = 1;
 		foreach (reverse @{$node->{array_size}}) {
-			my $oneOrMore = $self->{dom_doc}->createElement($self->{rng} . ":oneOrMore");
+			my $oneOrMore = $self->{dom_doc}->createElement($self->{rng} . "oneOrMore");
 			$current->appendChild($oneOrMore);
 
-			my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+			my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 			my $item = ($idx != 0) ? "item" . $idx : "item";
 			$element->setAttribute("name", $item);
 			$oneOrMore->appendChild($element);
@@ -559,6 +391,10 @@ sub visitTypeDeclarator {
 	} else {
 		if ($type->isa('SequenceType')) {
 			$type->visit($self, $define);
+		} elsif ($node->hasProperty("idref")) {
+			my $data = $self->{dom_doc}->createElement($self->{rng} . "data");
+			$data->setAttribute("type", "IDREF");
+			$define->appendChild($data);
 		} else {
 			$self->_ref_type($type, $define);
 		}
@@ -567,10 +403,6 @@ sub visitTypeDeclarator {
 	$self->_standalone($node, $dom_parent);
 }
 
-sub visitNativeType {
-	# empty
-}
-
 #
 #	3.11.1	Basic Types
 #
@@ -579,7 +411,7 @@ sub visitCharType {
 	my $self = shift;
 	my ($node, $dom_parent) = @_;
 
-	my $param = $self->{dom_doc}->createElement($self->{rng} . ":param");
+	my $param = $self->{dom_doc}->createElement($self->{rng} . "param");
 	$param->setAttribute("name", "length");
 	$dom_parent->appendChild($param);
 
@@ -613,13 +445,22 @@ sub visitStructType {
 		}
 	}
 
-	my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+	my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 	$define->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($define);
 
-	my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
+	my $group = $self->{dom_doc}->createElement($self->{rng} . "group");
 	$define->appendChild($group);
 
+	if ($node->hasProperty("id")) {
+		my $attribute = $self->{dom_doc}->createElement($self->{rng} . "attribute");
+		$attribute->setAttribute("name", "id");
+		$group->appendChild($attribute);
+		my $data = $self->{dom_doc}->createElement($self->{rng} . "data");
+		$data->setAttribute("type", "ID");
+		$attribute->appendChild($data);
+	}
+
 	foreach (@{$node->{list_member}}) {
 		$self->_get_defn($_)->visit($self, $group);
 	}
@@ -632,8 +473,17 @@ sub visitMember {
 	my ($node, $dom_parent) = @_;
 
 	my $type = $self->_get_defn($node->{type});
+	while ($type->isa('TypeDeclarator') and !exists $type->{array_size}) {
+		$type = $self->_get_defn($type->{type});
+	}
+	unless ($type->isa('SequenceType')) {
+		$type = $self->_get_defn($node->{type});
+	}
 
-	my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+	my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
+	if ($node->hasProperty("attr") and !exists $node->{array_size}) {
+		$element = $self->{dom_doc}->createElement($self->{rng} . "attribute");
+	}
 	$element->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($element);
 
@@ -647,10 +497,10 @@ sub visitMember {
 		}
 		my $first = 1;
 		foreach (reverse @{$node->{array_size}}) {
-			my $oneOrMore = $self->{dom_doc}->createElement($self->{rng} . ":oneOrMore");
+			my $oneOrMore = $self->{dom_doc}->createElement($self->{rng} . "oneOrMore");
 			$current->appendChild($oneOrMore);
 
-			my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+			my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 			my $item = ($idx != 0) ? "item" . $idx : "item";
 			$element->setAttribute("name", $item);
 			$oneOrMore->appendChild($element);
@@ -689,26 +539,26 @@ sub visitUnionType {
 
 	my $type = $self->_get_defn($node->{type});
 
-	my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+	my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 	$define->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($define);
 
-	my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
+	my $group = $self->{dom_doc}->createElement($self->{rng} . "group");
 	$define->appendChild($group);
 
-	my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+	my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 	$element->setAttribute("name", "discriminator");
 	$group->appendChild($element);
 
 	if ($type->isa('EnumType')) {
 		$type->visit($self, $element, 1);
 	} else {
-		my $data = $self->{dom_doc}->createElement($self->{rng} . ":data");
+		my $data = $self->{dom_doc}->createElement($self->{rng} . "data");
 		$data->setAttribute("type", $type->{xsd_name});
 		$element->appendChild($data);
 	}
 
-	my $choice = $self->{dom_doc}->createElement($self->{rng} . ":choice");
+	my $choice = $self->{dom_doc}->createElement($self->{rng} . "choice");
 	$group->appendChild($choice);
 
 	foreach (@{$node->{list_expr}}) {
@@ -742,17 +592,6 @@ sub visitCase {
 	$self->_get_defn($node->{element}->{value})->visit($self, $dom_parent);		# single or array
 }
 
-#	3.11.2.3	Constructed Recursive Types and Forward Declarations
-#
-
-sub visitForwardStructType {
-	# empty
-}
-
-sub visitForwardUnionType {
-	# empty
-}
-
 #	3.11.2.4	Enumerations
 #
 #	See	1.2.7.1		Enum
@@ -764,12 +603,12 @@ sub visitEnumType {
 	return if (exists $self->{done_hash}->{$node->{xsd_name}});
 	$self->{done_hash}->{$node->{xsd_name}} = 1;
 
-	my $choice = $self->{dom_doc}->createElement($self->{rng} . ":choice");
+	my $choice = $self->{dom_doc}->createElement($self->{rng} . "choice");
 
 	if ($indirect) {
 		$dom_parent->appendChild($choice);
 	} else {
-		my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+		my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 		$define->setAttribute("name", $node->{xsd_name});
 		$dom_parent->appendChild($define);
 
@@ -788,7 +627,7 @@ sub visitEnum {
 	my ($node, $dom_parent) = @_;
 	my $FH = $self->{out};
 
-	my $value = $self->{dom_doc}->createElement($self->{rng} . ":value");
+	my $value = $self->{dom_doc}->createElement($self->{rng} . "value");
 	$dom_parent->appendChild($value);
 
 	my $text = $self->{dom_doc}->createTextNode($node->{xsd_name});
@@ -813,10 +652,10 @@ sub visitSequenceType {
 		$current = $self->_get_defn($current->{type});
 	}
 
-	my $zeroOrMore = $self->{dom_doc}->createElement($self->{rng} . ":zeroOrMore");
+	my $zeroOrMore = $self->{dom_doc}->createElement($self->{rng} . "zeroOrMore");
 	$dom_parent->appendChild($zeroOrMore);
 
-	my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+	my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 	my $item = ($idx != 0) ? "item" . $idx : "item";
 	$element->setAttribute("name", $item);
 	$zeroOrMore->appendChild($element);
@@ -837,7 +676,7 @@ sub visitStringType {
 	my ($node, $dom_parent) = @_;
 
 	if (exists $node->{max}) {
-		my $param = $self->{dom_doc}->createElement($self->{rng} . ":param");
+		my $param = $self->{dom_doc}->createElement($self->{rng} . "param");
 		$param->setAttribute("name", "maxLength");
 		$dom_parent->appendChild($param);
 
@@ -862,14 +701,14 @@ sub visitFixedPtType {
 	my $self = shift;
 	my ($node, $dom_parent) = @_;
 
-	my $param = $self->{dom_doc}->createElement($self->{rng} . ":param");
+	my $param = $self->{dom_doc}->createElement($self->{rng} . "param");
 	$param->setAttribute("name", "totalDigits");
 	$dom_parent->appendChild($param);
 
 	my $value = $self->{dom_doc}->createTextNode($self->_value($node->{d}));
 	$param->appendChild($value);
 
-	$param = $self->{dom_doc}->createElement($self->{rng} . ":param");
+	$param = $self->{dom_doc}->createElement($self->{rng} . "param");
 	$param->setAttribute("name", "fractionDigits");
 	$dom_parent->appendChild($param);
 
@@ -901,19 +740,19 @@ sub visitException {
 		}
 	}
 
-	my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+	my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 	$define->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($define);
 
 	if (scalar @{$node->{list_member}}) {
-		my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
+		my $group = $self->{dom_doc}->createElement($self->{rng} . "group");
 		$define->appendChild($group);
 
 		foreach (@{$node->{list_member}}) {
 			$self->_get_defn($_)->visit($self, $group);
 		}
 	} else {
-		my $empty = $self->{dom_doc}->createElement($self->{rng} . ":empty");
+		my $empty = $self->{dom_doc}->createElement($self->{rng} . "empty");
 		$define->appendChild($empty);
 	}
 
@@ -933,11 +772,11 @@ sub visitOperation {
 	return unless ($self->{standalone});
 
 	if (scalar(@{$node->{list_in}}) + scalar(@{$node->{list_inout}})) {
-		my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+		my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 		$define->setAttribute("name", $node->{xsd_name});
 		$dom_parent->appendChild($define);
 
-		my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
+		my $group = $self->{dom_doc}->createElement($self->{rng} . "group");
 		$define->appendChild($group);
 
 		foreach (@{$node->{list_param}}) {	# parameter
@@ -951,15 +790,15 @@ sub visitOperation {
 	my $type = $self->_get_defn($node->{type});
 	if (scalar(@{$node->{list_inout}}) + scalar(@{$node->{list_out}})
 			or ! $type->isa('VoidType') ) {
-		my $define = $self->{dom_doc}->createElement($self->{rng} . ":define");
+		my $define = $self->{dom_doc}->createElement($self->{rng} . "define");
 		$define->setAttribute("name", $node->{xsd_name} . "Response");
 		$dom_parent->appendChild($define);
 
-		my $group = $self->{dom_doc}->createElement($self->{rng} . ":group");
+		my $group = $self->{dom_doc}->createElement($self->{rng} . "group");
 		$define->appendChild($group);
 
 		unless ($type->isa("VoidType")) {
-			my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+			my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 			$element->setAttribute("name", $node->{xsd_name});
 			$group->appendChild($element);
 
@@ -981,68 +820,67 @@ sub visitParameter {
 
 	my $type = $self->_get_defn($node->{type});
 
-	my $element = $self->{dom_doc}->createElement($self->{rng} . ":element");
+	my $element = $self->{dom_doc}->createElement($self->{rng} . "element");
 	$element->setAttribute("name", $node->{xsd_name});
 	$dom_parent->appendChild($element);
 
 	$self->_ref_type($type, $element);
 }
 
+##############################################################################
+
+package CORBA::XMLSchemas::relaxngImportVisitor;
+
+use base qw(CORBA::XMLSchemas::relaxngVisitor);
+
+sub new {
+	my $proto = shift;
+	my $class = ref($proto) || $proto;
+	my $self = {};
+	bless($self, $class);
+	my ($parser) = @_;
+	$self->{standalone} = undef;
+	$self->{tag_root} = "";
+	$self->{_rng} = 'rng';
+	$self->{rng} = 'rng:';
+	$self->{_xsd} = 'xs';
+	$self->{xsd} = 'xs:';
+	$self->{_corba} = 'corba';
+	$self->{corba} = 'corba:';
+	$self->{symbtab} = $parser->YYData->{symbtab};
+	$self->{root} = $parser->YYData->{root};
+	$self->{done_hash} = {};
+	$self->{num_key} = 'num_inc_rng';
+	$self->{need_corba} = undef;
+	return $self;
+}
+
 #
-#	3.14	Attribute Declaration
+#	3.5		OMG IDL Specification
 #
 
-sub visitAttributes {
+sub visitSpecification {
 	my $self = shift;
-	my ($node, $dom_parent) = @_;
+	my ($node, $dom_doc, $dom_parent) = @_;
+
+	$self->{dom_doc} = $dom_doc;
+
+	my $grammar = $self->{dom_doc}->createElement($self->{rng} . "grammar");
+	$grammar->setAttribute("ns", "http://www.omg.org/IDL-Mapped/");
+	$grammar->setAttribute("datatypeLibrary", "http://www.w3.org/2001/XMLSchema-datatypes");
+	$grammar->setAttribute("xmlns:" . $self->{_xsd}, "http://www.w3.org/2001/XMLSchema");
+	$grammar->setAttribute("xmlns:" . $self->{_rng}, "http://relaxng.org/ns/structure/1.0");
+	$grammar->setAttribute("xmlns:" . $self->{_corba}, "http://www.omg.org/IDL-WSDL/1.0/");
+	$dom_parent->appendChild($grammar);
+
+	if (exists $node->{list_import}) {
+		foreach (@{$node->{list_import}}) {
+			$_->visit($self, $grammar);
+		}
+	}
 	foreach (@{$node->{list_decl}}) {
-		$self->_get_defn($_)->visit($self, $dom_parent);
+		$self->_get_defn($_)->visit($self, $grammar);
 	}
 }
-
-sub visitAttribute {
-	my $self = shift;
-	my ($node, $dom_parent) = @_;
-	$node->{_get}->visit($self, $dom_parent);
-	$node->{_set}->visit($self, $dom_parent)
-			if (exists $node->{_set});
-}
-
-#
-#	3.15	Repository Identity Related Declarations
-#
-
-sub visitTypeId {
-	# empty
-}
-
-sub visitTypePrefix {
-	# empty
-}
-
-#
-#	3.16	Event Declaration
-#
-
-sub visitEvent {
-	shift->_no_mapping(@_);
-}
-
-#
-#	3.17	Component Declaration
-#
-
-sub visitComponent {
-	shift->_no_mapping(@_);
-}
-
-#
-#	3.18	Home Declaration
-#
-
-sub visitHome {
-	shift->_no_mapping(@_);
-}
-
 1;
 
